<?php

namespace Center\Capsule;

use Godruoyi\Snowflake\Snowflake;
use Center\Capsule\Services\Extra\PaginatorService;
use Illuminate\Support\ServiceProvider;

class CapsuleServiceProvider extends ServiceProvider
{
    /**
     * Perform post-registration booting of services.
     *
     * @return void
     */
    public function boot()
    {
        // 发布配置文件
        $this->publishes([
            __DIR__ . '/../config/capsule.php' => config_path('capsule.php'),
        ], 'capsule');
        // 注册路由
        if (method_exists($this->app, 'routesAreCached') && !$this->app->routesAreCached()) {
            require __DIR__ . '/routes.php';
        }
    }

    /**
     * Register any package services.
     *
     * @return void
     */
    public function register()
    {
        $this->app->bind(
            'Illuminate\Pagination\LengthAwarePaginator',
            function ($app, $options) {
                return new PaginatorService($options['items'], $options['total'], $options['perPage'], $options['currentPage'], $options['options']);
            });

        /* SnowFlake */
        $this->app->singleton('snowflake', function () {
            return (new Snowflake(config('dataCenter'), config('workerId')));
        });

    }

}