<?php

namespace Center\Capsule\Controllers\Models;


use Center\Capsule\Controllers\Controller;
use Center\Capsule\Controllers\Requests\ModelsRequest;
use Center\Capsule\Services\Models\ModelsService;

class ModelsController extends Controller
{
    protected $modelsService;

    public function __construct(ModelsService $modelsService)
    {
        $this->modelsService = $modelsService;
    }

    /**
     * 创建模型
     *
     * @param ModelsRequest $request
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function create(ModelsRequest $request)
    {
        $application_key = $request->input('application_key');
        $name            = $request->input('model_name');
        $res             = $this->modelsService->create($application_key, $name);
        if (!$res) {
            $this->errorReport('insert_model_fail');
        }

        $this->success($res);
    }

    /**
     * 更新模型
     *
     * @param ModelsRequest $request
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function update(ModelsRequest $request)
    {
        $data = [
            'application_key' => $request->input('application_key'),
            'name'            => $request->input('model_name'),
            'model_key'       => $request->input('model_key')
        ];
        $res  = $this->modelsService->update($data);
        if (!$res) {
            $this->errorReport('update_model_fail');
        }

        $this->success($res);
    }

    /**
     * 模型列表
     *
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function show()
    {
        $res = $this->modelsService->show(request()->all());

        $this->success($res);
    }

    /**
     * 删除模型
     *
     * @param $application_key
     * @param $model_key
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function delete($application_key, $model_key)
    {
        $res = $this->modelsService->delete($application_key, $model_key);
        if (!$res) {
            $this->errorReport('delete_model_fail');
        }

        $this->success($res);
    }

    /**
     * 获取模型属性
     *
     * @param ModelsRequest $request
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function getProperties(ModelsRequest $request)
    {
        $res = $this->modelsService->getProperties($request->all());

        $this->success($res);
    }

    /**
     *  获取模型详情
     *
     * @param ModelsRequest $request
     * @param               $model_key
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function detail(ModelsRequest $request, $model_key)
    {
        $application_key = $request->input('application_key');
        $res             = $this->modelsService->detail($application_key, $model_key);

        $this->success($res);
    }

    /**
     * 获取模型下单个属性详情
     *
     * @param $model_key
     * @param $property_key
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function propertyDetail($model_key, $property_key)
    {
        $res = $this->modelsService->propertyDetail($model_key, $property_key);

        $this->success($res);
    }
}