<?php

namespace Center\Capsule\Controllers\Property;


use Center\Capsule\Controllers\Controller;
use Center\Capsule\Controllers\Requests\PropertyRequest;
use Center\Capsule\Services\Property\Preset\PresetGetService;
use Center\Capsule\Services\Property\PropertyService;
use Center\Capsule\Services\Property\SortService;
use Exception;

class PropertyController extends Controller
{
    /**
     *  创建
     *
     * @param PropertyService $PropertyService
     * @param PropertyRequest $request
     * @param                 $model_key
     * @throws Exception
     */
    public function create(PropertyService $PropertyService, PropertyRequest $request, $model_key)
    {
        $service = null;
        try {
            $service = $PropertyService::getInstance()
                ->getPropertyByType($request->post('type'));
            $service->init(array_merge($request->post(), [
                'model_key' => $model_key
            ]))->bind()->bindModel();
        } catch (Exception $e) {
            $this->error($e->getCode(), $e->getMessage());
        }
        $this->success([
            "key" => $service->getKey()
        ]);
    }

    /**
     * 修改
     *
     * @param PropertyService $PropertyService
     * @param PropertyRequest         $request
     * @param                 $model_key
     * @param                 $property_key
     * @throws Exception
     */
    public function update(PropertyService $PropertyService, PropertyRequest $request, $model_key, $property_key)
    {
        try {
            $service = $PropertyService::getInstance()
                ->getPropertyByKey($property_key, $model_key);
            $service->modifyInit(array_merge($request->only(PropertyRequest::onlyField()),
                [
                'model_key' => $model_key,
                'key'       => $property_key
            ]))->bind()->update();
        } catch (Exception $e) {
            $this->error($e->getCode(), $e->getMessage());
        }
        $this->success();
    }

    /**
     * 删除
     *
     * @param PropertyService $PropertyService
     * @param PropertyRequest         $request
     * @param                 $model_key
     * @param                 $property_key
     * @throws Exception
     */
    public function delete(PropertyService $PropertyService, PropertyRequest $request, $model_key, $property_key)
    {
        $service = $PropertyService::getInstance()
            ->getPropertyByKey($property_key, $model_key);
        try {
            $service->init(array_merge($request->post(), [
                'model_key' => $model_key,
                'key'       => $property_key
            ]))->delete();
        } catch (Exception $e) {
            $this->error($e->getCode(), $e->getMessage());
        }
        $this->success();
    }

    /**
     * 获取预设
     *
     * @param PresetGetService $PresetGetService
     */
    public function preset(PresetGetService $PresetGetService)
    {
        $this->success([
            "properties" => $PresetGetService->getList()
        ]);
    }

    /**
     * sort
     *
     * @param SortService     $service
     * @param PropertyRequest $request
     * @throws \Rutorika\Sortable\SortableException
     */
    public function sort(SortService $service, PropertyRequest $request)
    {
        $service->sort($request->positionEntityId, $request->id);
        $this->success();
    }
}
