<?php

namespace Center\Capsule\Controllers\Record;


use Center\Capsule\Controllers\Controller;
use Center\Capsule\Controllers\Requests\RecordRequest;
use Center\Capsule\Services\Record\RecordService;


class RecordController extends Controller
{
    protected $service;

    public function __construct()
    {
        $this->service = new RecordService();
    }

    /**
     * index
     *
     * @param  RecordRequest  $request
     * @param               $model_key
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function index(RecordRequest $request, $model_key)
    {
        $result = $this->service
            ->instance('collect', function ($service) use ($model_key, $request) {
                $service->setModelKey($model_key)
                    ->injectData($request->all());
            })->getRows();
        $this->success($result);
    }


    /**
     * store
     *
     * @param  RecordRequest  $request
     * @param               $model_key
     */
    public function store(RecordRequest $request, $model_key)
    {
        $result = $this->service
            ->instance('insert', function ($service) use ($model_key, $request) {
                $service->setModelKey($model_key)
                    ->injectData($request->all());
            })->buildInsertData()->insert();
        $result ? $this->success() :
            $this->errorReport('INSERT RECORD FAILED');
    }

    /**
     * show
     *
     * @param               $model_key
     * @param  int  $id
     * @param  RecordRequest  $request
     */
    public function show($model_key, int $id, RecordRequest $request)
    {
        //try{
        $result = $this->service
            ->instance('collect', function ($service) use ($model_key, $id, $request) {
                $service->setModelKey($model_key)
                    ->injectData($request->all());
            })->setId($id)->getDetail();
        $this->success($result);
        //}catch (\Exception $e){
        //$this->error($request::ERROR_FORMAT_CODE, $e->getMessage());
        //}

    }


    /**
     * Update the specified resource in storage.
     *
     * @param               $model_key
     * @param  int  $id
     * @param  RecordRequest  $request
     */
    public function update($model_key, int $id, RecordRequest $request)
    {
        $result = $this->service
            ->instance('put', function ($service) use ($model_key, $request) {
                $service->setModelKey($model_key)
                    ->injectData($request->all());
            })->setId($id)->update();
        $result ? $this->success() :
            $this->errorReport('UPDATE RECORD FAILED');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param               $model_key
     * @param               $id
     * @param  RecordRequest  $request
     */
    public function destroy($model_key, $id, RecordRequest $request)
    {
        $result = $this->service
            ->instance('put', function ($service) use ($model_key, $request) {
                $service->setModelKey($model_key);
            })->setIds($id)->delete();

        ($result === false) ? $this->errorReport('DELETE RECORD FAILED') :
            $this->success();

    }

    /**
     * 排序
     *
     * @param               $model_key
     * @param  int  $id
     * @param  RecordRequest  $request
     */
    public function sort($model_key, int $id, RecordRequest $request)
    {
        $this->service
            ->instance('put', function ($service) use ($model_key, $request) {
                $service->setModelKey($model_key)
                    ->injectData([]);
            })->setId($id)->sort() ?
            $this->success() :
            $this->errorReport('SORT RECORD FAILED');;
    }
}
