<?php

namespace Center\Capsule\Controllers\Requests;


class ModelsRequest extends BaseRequest
{
    public function authorize()
    {
        return true;
    }

    public function rules()
    {
        switch ($this->route()->getName()) {
            case 'model@create':
                return [
                    'application_key' => 'required',
                    'model_name'      => 'required|max:50',
                ];
                break;
            case 'model@getProperties':
                return [
                    'application_key' => 'required',
                    'model_key'       => 'required'
                ];
                break;
            case 'model@update':
                return [
                    'application_key' => 'required',
                    'model_key'       => 'required',
                    'model_name'      => 'required|max:50'
                ];
                break;
            case 'model@detail':
                return [
                    'application_key' => 'required'
                ];
                break;
            default:
                break;
        }
    }

    public function messages()
    {
        switch ($this->route()->getName()) {
            case 'model@create':
                return [
                    'model_name.required'      => 'model_name_is_required',
                    'application_key.required' => 'application_key_is_required',
                    'model_name.max'           => 'model_name_is_max'
                ];
                break;
            case 'model@getProperties':
                return [
                    'model_key.required'       => 'model_key_is_required',
                    'application_key.required' => 'application_key_is_required'
                ];
                break;
            case 'model@update':
                return [
                    'model_key.required'       => 'model_key_is_required',
                    'application_key.required' => 'application_key_is_required',
                    'model_name.required'      => 'model_name_is_required',
                    'model_name.max'           => 'model_name_is_max'
                ];
                break;
            case 'model@detail':
                return [
                    'application_key.required' => 'application_key_is_required'
                ];
                break;
            default:
                break;
        }
    }
}
