<?php

namespace Center\Capsule\Models;

use Center\Capsule\Constants\PropertyConst;
use Center\Capsule\Services\User\UserService;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Schema;
use Illuminate\Support\Str;

/**
 * @method static Model|Collection|static[]|static|null find(int $id, array $columns = ['*'])
 * @method static Model|$this create(array $attributes = [])
 * @method static Model|Collection|static[]|static|null findOrFail($id, $columns = ['*'])
 * @see Model
 */
class BaseModel extends Model
{
    use SoftDeletes;

    protected $fillable = ['*'];

    protected $hidden = ['deleted_at'];

    public $dates = ['created_at', 'updated_at', 'deleted_at'];

    public $condition = [];

    /**
     * 自动写入字段
     *
     * @var array
     */
    protected $autoField = [
        'company_id'  => 'current_company_id',
        'created_uid' => 'id',
        'updated_uid' => 'id',
        'user_id'     => 'id'
    ];

    /**
     * 返回表名
     *
     * @return string
     */
    public function getTable()
    {
        if (empty($this->table)) {
            return $table = sprintf(
                "%s%s",
                config('database.prefix'),
                $this->table ?: Str::snake(class_basename($this))
            );
        }
        return $this->table;
    }

    /**
     * auto created_uid,updated_uid
     */
    public static function boot()
    {
        parent::boot();
        static::creating(function (BaseModel $model) {
            $fields = $model->autoField;
            foreach ($fields as $key => $field) {
                if (Schema::hasColumn($model->getTable(), $key) && $userField = static::getUserInfo($field)) {
                    $model->{$key} = $userField;
                }
            }
        });
        static::updating(function (BaseModel $model) {
            if (Schema::hasColumn($model->getTable(), 'updated_uid')) {
                $model->updated_uid = static::getUserInfo('id');
            }
        });
    }

    /**
     * getUserInfo
     *
     * @param null $key
     * @return array|null|string|bool
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function getUserInfo($key = null)
    {
        $data = UserService::getLoginUser();
        if (!empty($data[$key])) {
            return $data[$key];
        } elseif (is_null($key) && !empty($data)) {
            return $data;
        }
        return false;
    }

    /**
     * writeSystematic
     *
     * @param         $model_id
     */
    public function writeSystematic($model_id)
    {
        $fields = Models::$embedField;
        unset($fields['deleted_at']);
        foreach ($fields as $key => $val) {
            $data = [
                'property_key'  => $key,
                'property_type' => 'systematic',
                'model_id'      => $model_id,
                'property_name' => $val['name'],
                'visibility'    => PropertyConst::$visibility['show']['value'],
                'systematic'    => PropertyConst::$systematic['enable']['value']
            ];
            ModelProperties::query()->create($data);
        }
    }
}
