<?php

namespace Center\Capsule\Services\Extra;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Illuminate\Support\Facades\Log;
use Psr\Http\Message\ResponseInterface;

class RequestService
{

    private $client = null;
    /**
     * @var ResponseInterface
     */
    private $response = null;
    /**
     * @var int 超时时间 10s
     */
    public static $timeOut = 10;

    protected static $instance;

    public $responseContent = null;

    private function __construct()
    {
        $this->client = new Client([
            'timeout' => self::$timeOut
        ]);
    }

    /**
     * getContent
     *
     * @param  bool  $is_json
     * @return false|string
     */
    public function getContent($is_json = true)
    {
        if (empty($this->responseContent)) {
            $this->responseContent = $this->response->getBody()->getContents();
        }
        $this->responseContent = ($is_json && is_string($this->responseContent)) ?
            json_decode($this->responseContent,
                true) : $this->responseContent;
        Log::info('请求响应:', (array) $this->responseContent);
        return $this->responseContent;
    }

    /**
     * getResult
     *
     * @return null
     */
    public function getResult()
    {
        $response = $this->getContent();
        if (isset($response['result'])) {
            return $response['result'];
        }
        return null;
    }

    /**
     * isSuccess
     *
     * @return bool
     */
    public function isSuccess()
    {
        $this->responseContent = null;
        $response = $this->getContent();
        if(isset($response['error_code']) && ($response['error_code'] == 0 ||
                $response['error_code'] == 200)){
            return  true;
        }
        return false;
    }

    /**
     *  post请求
     *
     * @param  string  $url
     * @param  array  $data
     * @param  array  $headers
     * @return $this
     * @throws GuzzleException
     */
    public function post(string $url, array $data = [], array $headers = [])
    {
        $headers = array_merge(self::getHeader(), $headers);
        Log::info("请求头:", $headers);
        Log::info("post请求数据:", $data);
        $this->response = $this->client->request('POST', $url, [
            'json'    => $data,
            'headers' => $headers
        ]);
        return $this;
    }

    /**
     * delete请求
     *
     * @param  string  $url
     * @param $headers
     * @param  array  $data
     * @return $this
     * @throws GuzzleException
     */
    public function delete(string $url, array $headers = [], array $data = [])
    {
        $headers = self::getHeader() + $headers;
        Log::info("请求头:", $headers);
        Log::info("delete请求数据:", $data);
        $this->response = $this->client->request('DELETE', $url, [
            'json' => $data,
            'headers' => $headers
        ]);
        return $this;
    }

    /**
     * get请求
     *
     * @param  string  $url
     * @param  array  $data
     * @param  array  $headers
     * @return $this
     * @throws GuzzleException
     */
    public function get(string $url, array $data = [], array $headers = [])
    {
        $headers = self::getHeader() + $headers;
        Log::info("请求头:", $headers);
        Log::info("get请求数据:", $data);
        $this->response = $this->client->request('GET', $url, [
            'query'   => $data,
            'headers' => $headers
        ]);
        return $this;
    }

    /**
     * put请求
     *
     * @param  string  $url
     * @param  array  $data
     * @param  array  $headers
     * @return string
     * @throws GuzzleException
     */
    public function put(string $url, array $data = [], array $headers = [])
    {
        $headers = self::getHeader() + $headers;
        Log::info("请求头:", $headers);
        Log::info("put请求数据:", $data);
        $this->response = $this->client->request('PUT', $url, [
            'headers' => $headers,
            'json'    => $data
        ]);
        return $this;
    }

    /**
     * getHeader
     *
     * @return array
     */
    public function getHeader()
    {
        return [
            //"Content-type"  => "application/json;charset='utf-8",//application/json;charset='utf-8
            'AUTHORIZATION' => request()->header('AUTHORIZATION'),
            'X-COMPANY-ID'  => request()->header('X-COMPANY-ID'),
            'X-USER-ID'     => request()->header('X-USER-ID'),
            'X-API-SOURCE'  => request()->header('X-API-SOURCE')
        ];
    }

    /**
     * 单例获取
     *
     * @return static
     */
    public static function getInstance()
    {
        //静态绑定
        if (is_null(static::$instance)) {
            static::$instance = new static;
        }
        return static::$instance;
    }
}