<?php

namespace Center\Capsule\Services\Models\Facades\Resolve;

use Center\Capsule\Models\BaseModel;
use Center\Capsule\Models\ModelProperties;
use Center\Capsule\Models\Models;
use Center\Capsule\Services\Models\Exception\ModelsException;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;

class ModelMange
{
    public static $modelMap = [];

    public $modelKey = null;

    /**
     * getRelateModels
     *
     * @return array
     * @throws ModelsException
     */
    private function getRelateModels()
    {
        if (empty(static::$modelMap)) {
            $app_id = $this->getBelongAppId();
            $models = Models::query()
                ->where('application_id', $app_id)->get();
            if (!$models->isEmpty()) {
                foreach ($models as $v) {
                    static::$modelMap[$v['model_key']] = $v;
                }
            }
        }
        return static::$modelMap;
    }

    /**
     * getBelongAppId
     *
     * @return mixed
     * @throws ModelsException
     */
    private function getBelongAppId()
    {
        $application_id = Models::query()
            ->where('model_key', $this->modelKey)->value('application_id');
        if (empty($application_id)) {
            throw new ModelsException('application Exception');
        }
        return $application_id;
    }


    /**
     * getModel
     *
     * @param string $key
     * @return mixed
     * @throws ModelsException
     */
    public function getModel(string $key)
    {
        $this->modelKey = $key;
        $this->getRelateModels();
        if (empty(static::$modelMap[$key])) {
            throw new ModelsException('model Exception');
        }
        $model                            = static::$modelMap[$key]['namespace'] . '\\' . $key;
        static::$modelMap[$key]['object'] = new $model();
        return static::$modelMap[$key]['object'];
    }

    /**
     * getModel
     *
     * @param string $key
     * @return BaseModel
     * @throws ModelsException
     */
    public function getModelSingle(string $key)
    {
        if (isset(static::$modelMap[$key]['object']) &&
            static::$modelMap[$key]['object'] instanceof BaseModel) {
            return static::$modelMap[$key]['object'];
        } else {
            return $this->getModel($key);
        }
    }


    /**
     * 通过模型key模型下属性
     *
     * @param string $model_key
     * @return array
     */
    public function getProperties(string $model_key)
    {
        $properties   = ModelProperties::with(['models'])
            ->whereHas('models', function (Builder $query) use ($model_key) {
                $query->where('model_key', $model_key);
            })->get();
        $property_map = [];
        foreach ($properties as &$property) {
            unset($property['model']);
            $property_map[$property['property_key']] = $property;
        }
        unset($properties);
        return $property_map;
    }


    /**
     * 获取可以被修改的属性
     *
     * @param string $model_key
     * @return array|Builder|Model
     */
    public function getAllowProperties(string $model_key)
    {
        $properties = $this->getProperties($model_key);
        if ($properties) {
            foreach ($properties as $k => $v) {
                if (isset(Models::getEmbedField()[$k])) {
                    unset($properties[$k]);
                }
            }
        }
        return empty($properties) ? [] : $properties;
    }
}