<?php

namespace Center\Capsule\Services\Property\Build;


use Center\Capsule\Services\Property\Build\Alter\AttachmentAlter;
use Center\Capsule\Services\Property\Build\Alter\DateAlter;
use Center\Capsule\Services\Property\Build\Alter\JsonAlter;
use Center\Capsule\Services\Property\Build\Alter\LocateAlter;
use Center\Capsule\Services\Property\Build\Alter\NumberAlter;
use Center\Capsule\Services\Property\Build\Alter\RelationAlter;
use Center\Capsule\Services\Property\Build\Alter\TextAlter;
use Center\Capsule\Services\Property\Build\Alter\VarCharAlter;
use Center\Capsule\Services\Property\Exception\PropertyException;

class Alter
{
    protected static $instance;

    protected $properties = [];

    public static $enablePropertyAlterMap = [
        'varchar'    => VarCharAlter::class,
        'number'     => NumberAlter::class,
        'date'       => DateAlter::class,
        'relation'   => RelationAlter::class,
        'json'       => JsonAlter::class,
        'text'       => TextAlter::class,
        'locate'     => LocateAlter::class,
        'attachment' => AttachmentAlter::class,
    ];


    /**
     * getPropertyPool
     *
     * @return array
     */
    public function getPropertyAlterPool()
    {
        if (empty($this->properties)) {
            foreach (static::$enablePropertyAlterMap as $key => $obj) {
                $this->properties[$key] = new $obj();
            }
        }
        return $this->properties;
    }

    /**
     * getPropertyAlterByKey
     *
     * @param $key
     * @return NumberAlter|VarCharAlter|DateAlter|JsonAlter|RelationAlter|TextAlter|LocateAlter|AttachmentAlter
     * @throws \Exception
     */
    public function getPropertyAlterByKey($key)
    {
        if (!isset($this->properties[$key])) {
            $this->bindPropertyAlter($key);
        }
        return $this->properties[$key];
    }

    /**
     *  bindPropertyAlter
     *
     * @param $key
     * @throws \Exception
     */
    protected function bindPropertyAlter($key)
    {
        if (isset(static::$enablePropertyAlterMap[$key])) {
            $this->properties[$key] = new static::$enablePropertyAlterMap[$key]();
        } else {
            throw new PropertyException("{$key} is not exist");
        }
    }

    /**
     * 单例获取
     *
     * @return static
     */
    public static function getInstance()
    {
        //静态绑定
        if (is_null(static::$instance)) {
            static::$instance = new static;
        }
        return static::$instance;
    }

    public function getOriginData()
    {
    }
}