<?php

namespace Center\Capsule\Services\Property\Build\Alter;

use Center\Capsule\Services\Property\Build\Alter;
use Center\Capsule\Services\Property\Build\NumberProperty;
use Center\Capsule\Services\Property\Interfaces\AlterInterface;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class NumberAlter extends Alter implements AlterInterface
{
    /**
     * @var NumberProperty
     */
    public $property = null;

    public function setProperty(NumberProperty $property)
    {
        $this->property = $property;
        return $this;
    }

    public function getNumberMakeMap()
    {
        return [
            'decimal'  => function () {
                return $this->decimalType();
            },
            'int'      => function () {
                return $this->intType();
            },
            'tinyint'  => function () {
                return $this->intType();
            },
            'smallint' => function () {
                return $this->intType();
            }
        ];
    }

    public function make()
    {
        $type = $this->property->numberType;
        $map  = $this->getNumberMakeMap();
        if (isset($map[$type]) && $map[$type] instanceof \Closure) {
            $map[$type]();
        }
    }

    protected function intType()
    {
        Schema::table($this->property->modelKey, function (Blueprint $table) {
            $action = $this->property->getInterMap($this->property->numberType);
            $table->{$action}($this->property->getKey(), false, $this->property->unsigned)
                ->comment($this->property->name)
                ->change();
        });
    }

    protected function decimalType()
    {
        Schema::table($this->property->modelKey, function (Blueprint $table) {
            $table->addColumn('decimal', $this->property->getKey(), [
                'total'    => $this->property->point,
                'places'   => $this->property->digit,
                'unsigned' => $this->property->unsigned
            ])->comment($this->property->name)
                ->change();
        });
    }
}