<?php

namespace Center\Capsule\Services\Property\Build;


use Center\Capsule\Models\ModelProperties;
use Center\Capsule\Services\Property\Build\Traits\AlterTrait;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class AttachmentProperty extends BaseProperty
{
    use AlterTrait;
    public $type = 'attachment';

    /**
     * 绑定数据到当前对象
     *
     * @return mixed
     */
    public function bind()
    {
        return $this;
    }


    /**
     * 绑定指定模型上
     *
     * @return mixed
     */
    public function bindModel()
    {

        DB::transaction(function () {
            (new Model((new ModelProperties()), $this->modelKey, $this))
                ->writeDb(function () {
                    $this->make();
                })->checkTable();
        });
        return true;
    }

    /**
     * make
     */
    protected function make()
    {
        return Schema::table($this->modelKey, function (Blueprint $table) {
            $table = $table->json($this->getKey())
                ->nullable()
                ->comment($this->name);
            if ($this->alter) {
                $table->change();
            }
        });
    }

    /**
     * update
     *
     * @return bool
     */
    public function update()
    {
        DB::transaction(function () {
            $model = new ModelProperties();
            (new Model($this->getPropertyModelByKey($model), $this->modelKey, $this))
                ->updateDb(function () {
                    (new Alter())->getPropertyAlterByKey($this->type)
                        ->setProperty($this)
                        ->make()
                        ->make();
                })->checkTable();
        });
        return true;
    }
}