<?php

namespace Center\Capsule\Services\Property\Build;


use Center\Capsule\Models\ModelProperties;
use Center\Capsule\Services\Property\Build\Traits\AlterTrait;
use Center\Capsule\Services\Property\Exception\PropertyException;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Center\Capsule\Services\Property\Preset\Conf\LocateProperty as Conf;
use Illuminate\Support\Facades\Schema;

class LocateProperty extends BaseProperty
{
    use AlterTrait;
    public $type = 'locate';

    public $locateType = 'address';

    /**
     * @var array $sourceOption
     */
    private $sourceOption = [];


    /**
     * getLocateMakeMap
     *
     * @return array
     */
    public function getLocateMakeMap()
    {
        return [
            "address" => function () {
                return $this->addressType();
            },
            //经纬度
            "lng_lat" => function () {
                return $this->lngLatType();
            },
            "ip"      => function () {
                return $this->ipType();
            }
        ];
    }

    /**
     * addressType
     *
     * @return array
     */
    protected function addressType()
    {
        $this->locateType   = 'address';
        $this->sourceOption = Conf::getOptionType($this->locateType);
        unset($this->sourceOption['division_data']);
        return $this->sourceOption;

    }

    /**
     * lngLatType
     *
     * @return array
     */
    protected function lngLatType()
    {
        $this->locateType   = 'lng_lat';
        $this->sourceOption = Conf::getOptionType($this->locateType);
        //todo 经纬度正常范围
        return $this->sourceOption;
    }

    /**
     * ipType
     *
     * @return array
     */
    protected function ipType()
    {
        $this->locateType   = 'ip';
        $this->sourceOption = Conf::getOptionType($this->locateType);
        //todo 正确ip
        return $this->sourceOption;
    }


    /**
     * 绑定数据到当前对象
     *
     * @return $this|mixed
     * @throws PropertyException
     */
    public function bind()
    {
        $this->bindLocateType()->bindOption();
        return $this;
    }

    /**
     * bindOption
     *
     * @throws PropertyException
     */
    protected function bindOption()
    {
        foreach ($this->sourceOption as $k => $v) {
            if (!isset($this->option['type'][$this->locateType]) ||
                !array_key_exists($k, $this->option['type'][$this->locateType])){
                throw new PropertyException('current option exception');
            }
        }
    }

    /**
     * bindLocateType
     *
     * @return $this
     * @throws PropertyException
     */
    public function bindLocateType()
    {
        if (!empty($this->option['type'])) {
            $type = key($this->option['type']);
            $map  = $this->getLocateMakeMap();
            if (isset($map[$type]) && $map[$type] instanceof \Closure) {
                $map[$type]();
            } else {
                throw new PropertyException('No locate type has been specified or defined');
            }
        }
        return $this;
    }

    /**
     * bindModel
     *
     * @return mixed|void
     */
    public function bindModel()
    {
        DB::transaction(function () {
            (new Model((new ModelProperties()), $this->modelKey, $this))
                ->writeDb(function () {
                    $this->make();
                })->checkTable();
        });
    }

    /**
     * make
     *
     */
    public function make()
    {
        Schema::table($this->modelKey, function (Blueprint $table) {
            $table = $table->json($this->getKey())
                ->nullable(true)
                ->comment($this->name);
            if ($this->alter) {
                $table->change();
            }
        });
    }

    /**
     * update
     *
     * @return bool|mixed
     */
    public function update()
    {
        DB::transaction(function () {
            $model = new ModelProperties();
            (new Model($this->getPropertyModelByKey($model), $this->modelKey, $this))
                ->updateDb(function () {
                    (new Alter())->getPropertyAlterByKey($this->type)
                        ->setProperty($this)
                        ->make()
                        ->make();
                })->checkTable();
        });
        return true;
    }
}