<?php

namespace Center\Capsule\Services\Property\Build;


use Center\Capsule\Models\ModelProperties;
use Center\Capsule\Services\Property\Exception\PropertyException;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

/**
 * Class VarCharProperty
 *
 * @package Center\Capsule\Services\Property\Build
 */
class VarCharProperty extends BaseProperty
{

    public $type = 'varchar';

    public $stringNum = 255;

    public $varCharType = 'default';

    /**
     * 绑定数据到当前对象
     *
     * @return $this|mixed
     * @throws PropertyException
     */
    public function bind()
    {
        $this->bindVarCharType();
        return $this;
    }

    /**
     * bindVarCharType
     *
     * @throws PropertyException
     */
    public function bindVarCharType()
    {
        if (!empty($this->option['type'])) {
            $type              = key($this->option['type']);
            $this->varCharType = $type;
            $this->bindOption();
        }
    }

    /**
     * bindOption
     *
     * @throws PropertyException
     */
    protected function bindOption()
    {
        $this->bindOptionMax();
    }

    /**
     * bindOptionMax
     *
     * @throws PropertyException
     */
    protected function bindOptionMax()
    {

        if (isset($this->option['type'][$this->varCharType]['max'])) {
            $number = $this->option['type'][$this->varCharType]['max'];
            if(is_numeric($number)){
                $this->setStringNum($number);
            }else{
                throw new PropertyException('max is numeric');
            }
        }
    }

    /**
     * setStringNum
     *
     * @param int $number
     * @return $this
     * @throws PropertyException
     */
    protected function setStringNum(int $number)
    {

        if($number > 255){
            throw new PropertyException('max property is limit 255');
        }else{
            $this->stringNum =  $number;
        }
        return $this;
    }


    /**
     * 绑定指定模型上
     *
     * @return mixed
     */
    public function bindModel()
    {
        DB::transaction(function () {
            (new Model((new ModelProperties()), $this->modelKey, $this))
                ->writeDb(function () {
                    $this->make();
                })->checkTable();
        });
        return true;
    }

    protected function make()
    {
        Schema::table($this->modelKey, function (Blueprint $table) {
            $table->string($this->getKey(), $this->stringNum)
                ->comment($this->name)
                ->default('');
        });
    }

    /**
     * update
     *
     * @return bool
     */
    public function update()
    {
        DB::transaction(function () {
            $model = new ModelProperties();
            (new Model($this->getPropertyModelByKey($model), $this->modelKey, $this))
                ->updateDb(function () {
                    (new Alter())->getPropertyAlterByKey($this->type)
                        ->setProperty($this)->make();
                })->checkTable();
        });
        return true;
    }
}