<?php

namespace Center\Capsule\Services\Property\Preset;



use App\Models\Capsule\PresetProperties;
use Center\Capsule\Services\Property\Preset\Conf\AttachmentProperty;
use Center\Capsule\Services\Property\Preset\Conf\DateProperty;
use Center\Capsule\Services\Property\Preset\Conf\JsonProperty;
use Center\Capsule\Services\Property\Preset\Conf\LocateProperty;
use Center\Capsule\Services\Property\Preset\Conf\NumberProperty;
use Center\Capsule\Services\Property\Preset\Conf\RelationProperty;
use Center\Capsule\Services\Property\Preset\Conf\TextProperty;
use Center\Capsule\Services\Property\Preset\Conf\VarCharProperty;
use Carbon\Carbon;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class PresetConf
{

    /**
     * getConf
     *
     * @return array
     * @throws \Exception
     */
    public static function getConf()
    {
        return [
            VarCharProperty::conf(),
            NumberProperty::conf(),
            RelationProperty::conf(),
            DateProperty::conf(),
            AttachmentProperty::conf(),
            TextProperty::conf(),
            JsonProperty::conf(),
            LocateProperty::conf()
        ];
    }


    public static function dataTypeMap()
    {

        return [
            "varchar"  => function (&$table, $v) {
                /**
                 * @var Blueprint $table
                 */
                $table->string($v->property_key, 50)->default(uniqid());
            },
            "date"     => function (&$table, $v) {
                /**
                 * @var Blueprint $table
                 */
                $table->date($v->property_key)->default(Carbon::now()->toDateString());
            },
            "number"   => function (&$table, $v) {
                /**
                 * @var Blueprint $table
                 */
                $table->integer($v->property_key, 10)->default(mt_rand(10, 100));
            },
            "relation" => function (&$table, $v) {
                /**
                 * @var Blueprint $table $v->property_option
                 */
                $table->json($v->property_key);
            },

            "attachment" => function (&$table, $v) {
                /**
                 * @var Blueprint $table
                 * [
                 * "id" => 12,
                 * "url"=> "ad consequat",
                 * "filename"=> "cupidatat ut quis commodo",
                 * "ext"=> "non laborum",
                 * "size"=>81223017,
                 * "width"=> 20179130,
                 * "heigh"=>4343995,
                 * "small_thumb" => [
                 * "url"    => "laborum id exercitation in",
                 * "width"  => 12674561,
                 * "height" => 58591649
                 * ]
                 * ]
                 */
                $table->json($v->property_key);
            },
            "json"       => function (&$table, $v) {
                /**
                 * @var Blueprint $table $v->property_option
                 */
                $table->json($v->property_key);
            },
            "locate"       => function (&$table, $v) {
                /**
                 * @var Blueprint $table $v->property_option
                 */
                $table->json($v->property_key);
            },
        ];
    }

    /**
     * make
     *
     * @throws \Exception
     */
    public static function make()
    {
        foreach (static::getConf() as $v) {
            PresetProperties::query()->updateOrCreate([
                'property_type' => $v['type']
            ], [
                'property_key'    => uniqid('field'),
                'property_type'   => $v['type'],
                'property_name'   => $v['name'],
                'property_option' => $v['option'],
                'property_status' => PresetConst::$propertyStatus['enable']['value']
            ]);

        }
    }

    /**
     * makeModel
     *
     * @throws \Exception
     */
    public static function makeModel()
    {

        foreach (static::getConf() as $v) {
            ModelProperties::query()->updateOrCreate([
                'model_id'      => '98685055588630531',
                'property_type' => $v['type']
            ], [
                'property_key'    => uniqid('field'),
                'property_type'   => $v['type'],
                'property_name'   => $v['name'],
                'property_option' => $v['option'],
                'visibility'      => PropertyConst::$visibility['show']['value']
            ]);

        }
    }

    public static function makeTable()
    {
        DB::transaction(function () {
            $prefix     = config('database.connections.mysql.prefix');
            $map        = self::dataTypeMap();
            $model_keys = DB::table('models')->pluck('model_key', 'id');
            foreach ($model_keys as $model_id => $model_key) {
                Schema::table($model_key, function (Blueprint $table) use ($model_id, $map, $model_key, $prefix) {
                    $columns = Schema::getConnection()->getDoctrineSchemaManager()
                        ->listTableColumns($prefix . $model_key);
                    $v       = DB::table('model_properties')->where('model_id', $model_id)->get()->toArray();
                    foreach ($v as $val) {
                        if (empty($columns[$val->property_key])) {
                            if (isset($map[$val->property_type])) {
                                $map[$val->property_type]($table, $val);
                            }
                        }
                    }
                });
            }
        });
    }


}