<?php

namespace Center\Capsule\Services\Property;


use Center\Capsule\Models\ModelProperties;
use Center\Capsule\Services\Property\Build\AttachmentProperty;
use Center\Capsule\Services\Property\Build\BaseProperty;
use Center\Capsule\Services\Property\Build\DateProperty;
use Center\Capsule\Services\Property\Build\JsonProperty;
use Center\Capsule\Services\Property\Build\LocateProperty;
use Center\Capsule\Services\Property\Build\NumberProperty;
use Center\Capsule\Services\Property\Build\RelationProperty;
use Center\Capsule\Services\Property\Build\SystematicProperty;
use Center\Capsule\Services\Property\Build\TextProperty;
use Center\Capsule\Services\Property\Build\VarCharProperty;
use Center\Capsule\Services\Property\Exception\PropertyException;
use Illuminate\Database\Eloquent\Builder;

class PropertyService
{
    protected static $instance;

    protected $properties = [];

    /**
     * @var array service pool
     */
    public static $enablePropertyMap = [
        'varchar'    => VarCharProperty::class,
        'number'     => NumberProperty::class,
        'date'       => DateProperty::class,
        'relation'   => RelationProperty::class,
        'json'       => JsonProperty::class,
        'text'       => TextProperty::class,
        'attachment' => AttachmentProperty::class,
        'locate'     => LocateProperty::class,
        'systematic' => SystematicProperty::class
    ];


    /**
     * getPropertyPool
     *
     * @return array
     */
    public function getPropertyPool()
    {
        if (empty($this->properties)) {
            foreach (static::$enablePropertyMap as $key => $obj) {
                $this->properties[$key] = new $obj();
            }
        }
        return $this->properties;
    }

    /**
     * getPropertyByType
     *
     * @param $type
     * @return BaseProperty;
     * @throws \Exception
     */
    public function getPropertyByType($type)
    {
        if (!isset($this->properties[$type])) {
            $this->bindProperty($type);
        }
        return $this->properties[$type];
    }

    /**
     * getPropertyByKey
     *
     * @param $property_key
     * @param $model_key
     * @return mixed
     * @throws \Exception
     */
    public function getPropertyByKey(string $property_key, string $model_key)
    {
        $type = ModelProperties::with(['models'])
            ->whereHas('models', function (Builder $query) use ($model_key) {
                $query->where('model_key', $model_key);
            })->where('property_key', $property_key)
            ->value('property_type');
        if (!isset($this->properties[$type])) {
            $this->bindProperty($type);
        }
        return $this->properties[$type];
    }

    /**
     * bindProperty
     *
     * @param $key
     * @throws \Exception
     */
    protected function bindProperty($key)
    {
        if (isset(static::$enablePropertyMap[$key])) {
            $this->properties[$key] = new static::$enablePropertyMap[$key]();
        } else {
            throw new PropertyException("{$key} service is Illegal");
        }
    }

    /**
     * 单例获取
     *
     * @return static
     */
    public static function getInstance()
    {
        //静态绑定
        if (is_null(static::$instance)) {
            static::$instance = new static;
        }
        return static::$instance;
    }
}