<?php

namespace Center\Capsule\Services\Record\Resolve;


use Center\Capsule\Services\Models\Facades\Model;
use Center\Capsule\Services\Record\Traits\BuildTrait;
use Center\Capsule\Constants\ServiceConst;
use Center\Capsule\Services\Extra\RequestService;
use Center\Capsule\Services\Record\Traits\TransTrait;
use GuzzleHttp\Exception\GuzzleException;
use Illuminate\Contracts\Pagination\LengthAwarePaginator;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use zyimm\query\build\QueryWhere;

class Collect
{
    use BuildTrait, TransTrait;

    private $id = 0;

    private $select;

    /* 过滤器 */
    private $filters;

    /* 排序器 */
    private $sorts;

    public $perPage = 10;

    public $lastSql = '';

    private $condition = [
        '=' => [
            'company_id'
        ]
    ];

    public function setId($id)
    {
        $this->id = $id;
        return $this;
    }

    public function setSelect()
    {
        $this->select = array_keys(Model::getProperties($this->modelKey));
        return $this;
    }

    public function setPerPage()
    {
        $page          = static::$request->get('count', $this->perPage);
        $this->perPage = $page;
        return $this;
    }

    /**
     * setSort
     *
     */
    public function setSort()
    {
        $sort = static::$request->get('sort');
        if(!empty($sort) && $sorts = explode(',', $sort)){
            foreach ($sorts as $sort){
                list($field, $per) = explode('.', $sort);
                if(isset($field) && isset($per)){
                    $this->sorts[] = [
                        $field, $per
                    ];
                }
            }
        }
    }


    public function setConditionRule(array $condition)
    {
        $this->condition = array_merge($this->condition, $condition);
        return $this;
    }

    /**
     * getCondition
     *
     * @param  Builder
     */
    public function getCondition(&$Builder)
    {
        if ($this->companyId) {
            $this->condition['='][]         = 'company_id';
            $this->injectData['company_id'] = $this->companyId;
        }

        if (!empty($this->id)) {
            $this->condition['='][] = 'id';
            $this->injectData['id'] = $this->id;
        }
        //自定义配置查询
        $this->condition = array_merge_recursive($this->condition, (array) $this->model->condition);
        /**
         * @var QueryWhere $build
         */
        $build = app('QueryWhere');
        $build->buildQueryWhere($this->injectData, $this->condition, $Builder);
    }


    /**
     * getDetail
     *
     * @return array
     * @throws GuzzleException
     * @throws \Exception
     */
    public function getDetail()
    {
        $this->isGet = true;
        $this->setSelect();
        $query = $this->model::query()->select($this->select)
            ->where(function ($query) {
                $this->getCondition($query);
            });

        $data = $query->first();
        if (!empty($data)) {
            foreach ($data->toArray() as $k => $v) {
                $data->$k = $this->transValues($v, $k);
            }
            $data = $this->mergeUserInfo([$data->toArray()]);
        } else {
            $data = [];
        }
        return reset($data);
    }

    /**
     * getRows
     *
     * @return Builder[]|Collection|mixed
     * @throws GuzzleException
     */
    public function getRows()
    {
        $this->isGet = true;
        DB::enableQueryLog();
        $this->setPerPage()->setSelect()->setSort();
        $queries       = $this->model::query()->select($this->select)
            ->where(function ($query) {
                $this->getCondition($query);
            });
        $queries       = $this->orderBy($queries)->paginate($this->perPage);
        $this->lastSql = DB::getQueryLog();
        $data          = $this->toArray($queries);
        foreach ($data as &$v) {
            foreach ($v as $property => &$value) {
                $value = $this->transValues($value, $property);
            }
        }
        $data = $this->mergeUserInfo($data);
        if (static::$request->get('dd')) {
            dd($this->lastSql);
        }
        return [
            'data'      => $data,
            'page_info' => [
                'total' => $queries->total(),
                'count' => $queries->perPage(),
                'page'  => $queries->currentPage()
            ]
        ];
    }

    /**
     * toArray
     *
     * @param  LengthAwarePaginator  $queries
     * @return array|mixed
     * @throws GuzzleException
     */
    protected function toArray(LengthAwarePaginator $queries)
    {
        $data = [];
        if (!$queries->isEmpty()) {
            foreach ($queries->items() as $item) {
                $data[] = $item->toArray();
            }
        }
        return $data;
    }

    /**
     * 追加用户信息
     *
     * @param  array  $data
     * @return array
     * @throws \Exception
     */
    private function mergeUserInfo(array $data = [])
    {
        $this->getUsers($data);
        if (!empty($this->users)) {
            try {
                $host     = ServiceConst::getServiceHost('user');
                $uri      = $host.config('capsule.user.get_users_info');
                $response = RequestService::getInstance()->post($uri, [
                    'ids' => array_unique($this->users)
                ]);
                if ($response->isSuccess()) {
                    $user_info = $response->getResult();
                    $users     = [];
                    foreach ($user_info as $v) {
                        $users[$v['user_id']] = $v;
                    }
                    foreach ($data as &$v) {
                        foreach ($v as $k => $val) {
                            if (in_array($k, ['user_id', 'created_uid', 'updated_uid']) && !empty($val)) {
                                if (isset($v[$k]) && isset($users[$v[$k]])) {
                                    $alias_k     = $k.'_info';
                                    $v[$alias_k] = $users[$v[$k]];
                                }
                            }
                        }
                    }
                }
            } catch (GuzzleException $e) {
                Log::error($e->getMessage());
            }
        }
        return $data;
    }

    /**
     * getUsers
     *
     * @param  array  $data
     */
    public function getUsers(array $data = [])
    {
        foreach ($data as $v) {
            foreach ($v as $k => $val) {
                if (in_array($k, ['created_uid', 'updated_uid']) && !empty($val)) {
                    $this->users[] = $val;
                }
            }
        }
    }


    /**
     * 排序
     *
     * @param $queries
     * @return Builder
     */
    private function orderBy(Builder $queries)
    {
        //自定义排序
        if (!empty($this->sorts)) {
            foreach ($this->sorts as $order) {
                list($field, $order) = $order;
                if(isset($field) && isset($per)){
                    $queries->orderBy($field, $order);
                }
            }
        }
        $queries->orderBy('id', 'desc');
        return $queries;
    }
}