<?php

use Illuminate\Support\Facades\Route;

Route::group(['prefix' => ''], function () {
    Route::post('/handle', 'Center\\Capsule\Controllers\HandleController@do');
    //属性路由
    Route::group([
        'namespace' => 'Center\\Capsule\Controllers\\Property',
        'prefix'    => 'property'
    ], function () {
        //获取预设属性
        Route::get('/preset', 'PropertyController@preset')
            ->name('property@preset');
        Route::post('/{model_key}', 'PropertyController@create')
            ->name('property@create');
        Route::put('/{model_key}/{property_key}', 'PropertyController@update')
            ->name('property@update');
        Route::delete('/{model_key}/{property_key}', 'PropertyController@delete')
            ->name('property@delete');
        Route::post('/sort/{id}', 'PropertyController@sort')
            ->name('property@sort');
    });

    //模型管理路由
    Route::group([
        'namespace' => 'Center\\Capsule\Controllers\\Models',
        'prefix'    => 'model'
    ], function () {
        Route::post('/', 'ModelsController@create')->name('model@create');
        Route::put('/', 'ModelsController@update')->name('model@update');
        Route::get('/', 'ModelsController@show')->name('model@show');
        Route::delete('/{application_key}/{model_key}', 'ModelsController@delete')->name('model@delete');
        Route::get('/properties', 'ModelsController@getProperties')->name('model@getProperties');
        Route::get('/detail/{model_key}', 'ModelsController@detail')->name('model@detail');
        Route::get('/{model_key}/property/{property_key}',
            'ModelsController@propertyDetail')->name('model@propertyDetail');
    });

    //记录管理路由
    Route::group([
        'namespace'  => 'Center\\Capsule\Controllers\\Record',
        'prefix'     => 'record'
    ], function () {
        Route::get('/{model_key}', 'RecordController@index')->name('record@index');
        Route::get('/{model_key}/{id}', 'RecordController@show')->name('record@show')->where('id', '[0-9]+');
        Route::post('/{model_key}', 'RecordController@store')->name('record@store');
        Route::put('/{model_key}/{id}', 'RecordController@update')->name('record@update')->where('id', '[0-9]+');
        Route::delete('/{model_key}/{id}', 'RecordController@destroy')->name('record@destroy');
        Route::post('/sort/{model_key}/{id}', 'RecordController@sort')->name('record@sort')->where('id', '[0-9]+');
    });
});